% Baseline New Keynesian Model as in the lecture, Gal (2008)
% Only difference: The alpha in Gal is here set to 0
% Macroeconomics, Master's Program, University of Bonn
% Zeno Enders

% Clears memory, command window and closes figures.
clear all; clc; close all;

% Makes the figures fullscreen
scz=get(0,'ScreenSize');
set(0,'DefaultFigurePosition',[10 0 scz(3) scz(4)-50])

%%%%%%%%%%%%%% IMPULSE RESPONSE OPTIONS %%%%%%%%%%%%%%%%%

% Number of periods to be plotted
npts   = 30;
% Number of columns of the figure
Columns= 3;

% Size of the shocks for impulse response
asize = 1;                 % Size of technology shock
nusize= 1;                 % Size of monetary shock

%%%%%%%%%%%%%%%%%%%%% PARAMETERS %%%%%%%%%%%%%%%%%%%%%%%
% Monetary Policy
rho_nu   = 0.5;           % Autocorrelation of mon. pol. shock
phi_y    = 0.5/4;         % Reaction of interest rate on output gap (in quarterly terms)
phi_pi   = 1.5;           % Reaction of interest rate on inflation

% Preferences
beta      = 0.99;         % Discount factor
sigma     = 1;            % Determines marginal utility of consumption
varphi    = 1;            % Determines marginal disutility of working  

% Calvo Paramter
theta     = 2/3;        

% Persistence of technology shocks
rho_a   = 0.9;

% Parameters that result from the above
lambda = (1-theta)*(1-beta*theta)/theta;
kappa  = lambda*sigma;
psi    = (1+varphi)/(sigma+varphi); % In Gal this parameter is called psi_ya^n

%%%%%%%%%%%%%% DECLARATION OF VARIABLES %%%%%%%%%%%%%%%%%%%%
num_var=1;
% First, declare the state variables
a      = num_var;      namevec='Technology';                                 num_var = num_var +1;
nu     = num_var;      namevec=strvcat(namevec,'Monetary Policy Shock');

nstates =num_var;   % Number of states
num_var= num_var+1; % Number of variables

%Now the jump variables
pi     = num_var;      namevec=strvcat(namevec,'Inflation');                num_var = num_var +1;
yg     = num_var;      namevec=strvcat(namevec,'Output Gap');               num_var = num_var +1;
y      = num_var;      namevec=strvcat(namevec,'Output');                   num_var = num_var +1;
h      = num_var;      namevec=strvcat(namevec,'Hours');                    num_var = num_var +1;
yn     = num_var;      namevec=strvcat(namevec,'Natural Rate of Output');   num_var = num_var +1;
rn     = num_var;      namevec=strvcat(namevec,'Natural Rate of Interest'); num_var = num_var +1;
i      = num_var;      namevec=strvcat(namevec,'Nominal Interest Rate');    num_var = num_var +1;
rr     = num_var;      namevec=strvcat(namevec,'Real Interest Rate');

A = zeros(num_var,num_var);  B = zeros(num_var,num_var);
%%%%%%%%%%%%%%%%%%%%%%% EQUATIONS %%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% Summary Equations of the Economy %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Natural rate of interest          Eq. 4.49
B(1,rn) = 1;
A(1,a)  = sigma*psi;
B(1,a)  = sigma*psi;

% New Keynesian Phillips Curve      Eq. 4.50
B(2,pi) = 1;
A(2,pi) = beta;
B(2,yg) = -kappa;

% Dynamic IS Curve                  Eq. 4.51
B(3,yg)= 1;
B(3,i) = 1/sigma;
A(3,pi)= 1/sigma;
B(3,rn)= -1/sigma;
A(3,yg)= 1;

% Taylor Rule                       Eq. 4.52
B(4,i) = -1;
B(4,pi)= phi_pi;
B(4,yg)= phi_y;
B(4,nu)= 1;

% Real Interest Rate                Eq. 4.16
B(5,rr)= -1;
B(5,i) = 1;
A(5,pi)= 1;

% Natural Rate of Output            Eq. 4.43
B(6,yn)=-1;
B(6,a) =psi;

% Output                            Eq. 4.45
B(7,y)  = -1;
B(7,yn) = 1;
B(7,yg) = 1;

% Hours                             Eq. 4.39
B(8,h)  =-1;
B(8,y)  = 1;
B(8,a)  = -1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%% Exogenous Processes  %%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Technology                       Eq. 4.60
A(9,a) = 1;
B(9,a) = rho_a;

% Monetary Policy                  Eq. 4.57
A(10,nu) = 1;
B(10,nu) = rho_nu;


%%%%%%%%%%%%%%%%%%%%%%% SOLVING THE MODEL %%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Using modified Klein's (2000) solution algorithm
[f,p] = my_solab(A,B,nstates);
% This program gives us the solution the form:
% States_t=p*States_(t-1)
% Controls_t=f*States_t

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%% IMPULSE RESPONSE FUNCTIONS %%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

choice = menu('Choose a Shock:', 'Technology', 'Monetary Policy', 'Both');
switch choice
    case 1
        shockchosen=a;
    case 2
         shockchosen=nu;
    case 3
        shockchosen=a+nu;
end

% Variables to plot
if shockchosen==a,
varlist=[a i rr y yg pi rn yn h];
elseif shockchosen==nu
varlist=[nu i rr y yg pi rn yn h];
else
varlist=[a nu i y yg pi rn yn h];
end

% Generating vector with the shock
vector=zeros(1,nstates);
if shockchosen==a,
    vector(a)=asize;
elseif shockchosen==nu,
    vector(nu)=nusize;
elseif shockchosen==a+nu,
    vector(a)=asize;
    vector(nu)=nusize;
end

% Calculating the impulse responses
R1=zeros(nstates,npts);
R2=zeros(num_var-nstates,npts);
R1(:,1)=(vector)';
R2(:,1)=f*R1(:,1);
for pt=2:npts
    R1(:,pt)=(p*R1(:,pt-1));
    R2(:,pt)=(f*R1(:,pt));
end
resp=[R1' R2'];

% Annualizing the interest rates and inflation
resp(:,i)=4*resp(:,i);
resp(:,rn)=4*resp(:,rn);
resp(:,pi)=4*resp(:,pi);
resp(:,rr)=4*resp(:,rr);

% Plotting the impulse-responses
jm=1;
figure('Color',[1 1 1])
for kk=varlist;
    subplot(ceil(length(varlist)/Columns),Columns,jm)
    plot(0:1:npts-1,zeros(npts),'k:',0:1:npts-1,resp(:,kk),'b-d','LineWidth',2,'MarkerSize',1);
    set(gca,'FontSize',12)
    title(deblank(namevec(kk,:)),'Fontsize',12);
    jm=jm+1;
end