% An example of the NKM
% We use here our D , f , g and z matrices (follow the slides)
% We get IRF, Time Series, Autocorrelation and Cross-Correlation

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%The model in structural form
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% x{t}=E{t}x{t+1}-(1/sigma)(i{t}-E{t}pi{t+1}r_n{t})% IS function
% pi{t}=beta*E{t}pi{t+1}+kappa*x{t}; The AS function
% i{t}=phi_pi*pi{t}+phi_x*x{t}; A simple Monetary Policy Rule
% r_n{t}=rho*r_n{t-1})+e{t} % r_n stands for the natural interest rate
% kappa=(1-theta)*(1-theta*beta))/(theta*(sigma+phi); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
clc

% calibration
beta=0.99;
sigma=1;
psi=1;
theta=2/3;
phi_x=0.5;
phi_pi=1.5;
rho=0.8;
kappa=(1-theta)*(1-theta*beta)/theta*(sigma+psi)

n=1; % number of pre-determined variables
m=2; % number of forward-looking variables

% Write down the matrices A,B,C and the  

A=[1,0,0;
    0,beta,0;
    1/sigma,1/sigma,1];

B=[rho,0,0;
    0,1,-kappa;
    0,phi_pi/sigma,(1+phi_x/sigma)];

C =[1;0;0];

H=inv(A)*B;
L=inv(A)*C;

% The Jordan decomposition of H

[p,lambda]= eig(H) % get eigenvectors and eigenvalues

% Sort eigenvalues and eigenvectors in ascending order

val=diag(lambda) % vectorize lambda
t=sortrows([val p'],1)
lambda=diag(t(:,1))
p=t(:,2:4)'
pstar=inv(p)

% Partition matrices

LAMBDA1=lambda(1:n,1:n) %(1:1,1;1)for the predetermined variable
LAMBDA2=lambda(n+1:m+n,n+1:m+n) %(2:3,2:3) for the jump variables

% partition matrices

P11=pstar(1:n,1:n)% (1:1,1:1)linha 1, coluna 1
P12=pstar(1:n,n+1:m+n)%(1:1,2:3) linha 1, colunas 2 e 3
P21=pstar(n+1:m+n,1:n)% (2:3,1:1)linhas 2 e 3, coluna 1
P22=pstar(n+1:m+n,n+1:m+n)% (2:3,2:3)linhas 2 e 3, colunas 2 e 3


R=pstar*L

% Calculate our D , f , g and z matrices

D=P11-P12*inv(P22)*P21
f=-inv(P22)*P21
R1=R(1:n,1:n)% Lines 1, column 1; %Notice that lines 2&3 are related to R2,not part of the solution
g=inv(D)*R1
z=inv(D)*LAMBDA1*D

% Calculate impulse response functions

h=30;
irf_r=zeros(3,h);

% w(t)=g*v(t)

irf_r(1,1)=g*[1];%Shock upon v

% w(t+1)=z*w(t). 
%Notice that from i+1 onwards shocks on v are zero so the term 
% g*shock disapears from the following code

i=1;
for i=1:(h-1);
    irf_r(1,i+1)=z*irf_r(1,i); 
end;

% Now let's get the IRF of y(t).
% Just apply the solution y(t)=f*w(t)

irf_r(2:3,:)=f*irf_r(1,:);

% In case of complex eigenvalues, consider only their real part

irf_r=real(irf_r);

% Impulse response functions from a unit shock upon r(t)

r_r=irf_r(1,:);
pi_r=irf_r(2,:);
x_r=irf_r(3,:);

% Notice that we can now calculate the IRF of i(t) easily as the Taylor
% rule considered in the model follows the next equation

i_r=phi_pi*pi_r+phi_x*x_r;

figure

subplot(121);
plot(r_r,'-*');
legend('r(t)'); grid on;
title('A unit shock upon r(t)')

subplot(122);
plot(x_r,'-*r');
hold on
plot(pi_r,'-*');
hold on
plot(i_r,'-o');
legend('x(t)','pi(t)','i(t)'); 
grid on;
hold off


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Autocorrelation and Cross-Correlation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%variable
%fi
lag=0:20;
figure
subplot(221);plot(lag,autocorr(x_r,20),'o-b'),grid on;title('Autocorrelation: x(t)')
subplot(222);plot(lag,autocorr(pi_r,20),'*-g'),grid on;title('Autocorrelation: pi(t)')
subplot(223);plot(lag,autocorr(i_r,20),'+-r'),grid on;title('Autocorrelation: i(t)')
subplot(224);plot(lag,autocorr(r_r,20),'p-k'),grid on;title('Autocorrelation: r(t)')
    
        
%Now let's go for the crosscorrelation between each variable and Output
%(Y). Notice that Yxx against Yxx (subplot 223)gives autocorrelation of
    
lags=-20:20;
    
figure
     
subplot(221);plot(lags,crosscorr(x_r,x_r,20),'o-b'),grid on;title('Cross-Correlation: x vs x')
subplot(222);plot(lags,crosscorr(pi_r,x_r,20),'*-g'),grid on;title('Cross-Correlation: pi vs x')
subplot(223);plot(lags,crosscorr(i_r,x_r,20),'+-r'),grid on; title('Cross-Correlation: i vs x')
subplot(224);plot(lags,crosscorr(r_r,x_r,20),'p-k'),grid on; title('Cross-Correlation: r vs x')
  
    
%Now, let's finish with calculating the Standard DEviation of each
%variable.
sigmax_r=std(x_r)
sigmapi_r=std(pi_r)
sigmai_r=std(i_r)
sigmar_r=std(r_r)
        
       
figure
y_y=[sigmax_r sigmapi_r sigmai_r sigmar_r];
names={'x';'pi';'i';'r'}
bar(y_y)
set(gca,'xticklabel',names)
title('Standard Deviation')
grid on

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Permanent shocks upon r(t)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

j =400; % number of periods
e=randn(1,j)*0.01; % draw shocks from normal distribution
irf_r=zeros(3,j);

% w(t)=g*v(t)

irf_r(1,1)=g*e(1,1);%Shock upon v

% w(t+1)=z*w(t). 
%Notice that from i+1 onwards shocks on v are zero so the term 
% g*shock disapears from the following code

i=1;
for i=1:(j-1);
    irf_r(1,i+1)=z*irf_r(1,i)+g*e(i+1); 
end;

% Now let's get the IRF of y(t).
% Just apply the solution y(t)=f*w(t)

irf_r(2:3,:)=f*irf_r(1,:);

% In case of complex eigenvalues, consider only their real part

irf_r=real(irf_r);

% Impulse response functions from a unit shock upon r(t)

r_r=irf_r(1,:);
pi_r=irf_r(2,:);
x_r=irf_r(3,:);

% Notice that we can now calculate the IRF of i(t) easily as the Taylor
% rule considered in the model follows the next equation

i_r=phi_pi*pi_r+phi_x*x_r;

figure
plot(r_r,'-b');
legend('r(t)'); grid on;
hold on
plot(i_r,'-r');
legend('r(t)','i(t)');
title('Permanent shocks upon r(t)')
grid on
hold off

figure
plot(x_r,'-r');
hold on
plot(pi_r,'-b');
hold on
legend('x(t)','pi(t)');
title('Permanent shocks upon r(t)')
grid on;
hold off






