% The New Keynesian Model
% Supply skocks plus demand shocks through the nominal interest rate
% These shocks are simulated separately
% A simple interest rate rule: i(t)depends only upon current inflation and
% shocks
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% x{t}=E{t}x{t+1}-(1/sigma)(i{t}-E{t}pi{t+1}); the traditional IS function
% pi{t}=beta*E{t}pi{t+1}+kappa*x{t}+u{t} ; the traditional AS functon
% i{t}=delta*pi{t}+v{t}; a very simplistic Monetary Policy Rule
% v{t+1}=rho_v*v{t}+e_v{t+1}; an aggregate demand shock;
% u{t+1}=rho_u*u{t}+e_u{t+1}; an AS shock
% kappa=(((1-omega)(1-beta*omega))/(alpha*omega)) ; 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
clc

% Calibrated parameter values

beta=0.99;
sigma=1;
omega=0.5;
alpha=3;
delta=1.5;
rho_v=0.5;
rho_u=0.8;
kappa=(1-omega)*(1-beta*omega)/(alpha*omega);

% Define state space matrices

A0=zeros(4,4);
A0(1,1)=1;
A0(2,2)=1;
A0(3,3)=1;
A0(3,4)=sigma^-1;
A0(4,4)=beta;

A1=zeros(4,4);
A1(1,1)=rho_v;
A1(2,2)=rho_u;
A1(3,1)=sigma^-1;
A1(3,3)=1;
A1(3,4)=sigma^-1*delta;
A1(4,2)=-1;
A1(4,3)=-kappa;
A1(4,4)=1;

B0=zeros(4,2);
B0(1,1)=1;
B0(2,2)=1;

% Calculate alternative state space matrices

A=inv(A0)*A1;
B=inv(A0)*B0;

% The Jordan decomposition of A

[p,lambda]=eig(A)

% Sort eigenvalues and eigenvectors in ascending order

val=diag(lambda)
t=sortrows([val p'],1)
lambda=diag(t(:,1))
p=t(:,2:5)'
pstar=inv(p)

% Partition matrices

LAMBDA1=lambda(1:2,1:2)
LAMBDA2=lambda(3:4,3:4)

P11=pstar(1:2,1:2)
P12=pstar(1:2,3:4)
P21=pstar(3:4,1:2)
P22=pstar(3:4,3:4)

R=pstar*B

% Calculate our D , f , g and z matrices

D=P11-P12*inv(P22)*P21;
f=-inv(P22)*P21;
R1=R(1:2,:) % Lines 1&2, all entries; 
%Notice that lines 3&4 are related to R2,not part of the solution
g=inv(D)*R1;
z=inv(D)*LAMBDA1*D;

% Calculate impulse response functions; the two shocks will be analysed
% separately

h=20;
irf_v=zeros(4,h);
irf_u=zeros(4,h);


% w(t)=g*v(t)

irf_v(1:2,1)=g*[1;0];%Shock upon v
irf_u(1:2,1)=g*[0;1];%Schock upon u

% w(t+1)=z*w(t). 
%Notice that from i+1 onwards shocks on v and u are zero so the therm 
% g*shock disapears from the following code

i=1;
for i=1:(h-1);
    irf_v(1:2,i+1)=z*irf_v(1:2,i); 
    irf_u(1:2,i+1)=z*irf_u(1:2,i);
end;

% Now let's get the IRF of y(t).
% Just apply the solution y(t)=f*w(t)

irf_u(3:4,:)=f*irf_u(1:2,:)
irf_v(3:4,:)=f*irf_v(1:2,:);

% In case of complex eigenvalues, consider only their real part

irf_v=real(irf_v);
irf_u=real(irf_u);

% Impulse response functions from a unit shock upon v(t)

v_v=irf_v(1,:);
u_v=irf_v(2,:);
x_v=irf_v(3,:);
pi_v=irf_v(4,:);
figure
subplot(221);plot(v_v,'-*');
legend('v(t)'); grid on;
title('A unit shock upon v(t)')
subplot(222);plot(u_v,'-*');
legend('u(t)'); grid on;
subplot(223);plot(x_v,'-o');
legend('x(t)'); grid on;
subplot(224);plot(pi_v,'-o');
legend('pi(t)'); grid on;


% Impulse response functions from a unit shock upon u(t)

v_u=irf_u(1,:);
u_u=irf_u(2,:);
x_u=irf_u(3,:);
pi_u=irf_u(4,:);
figure
subplot(221);plot(v_u,'-*r');
legend('v(t)'); grid on;
title('A unit shock upon u(t)')
subplot(222);plot(u_u,'-*r');
legend('u(t)'); grid on;
subplot(223);plot(x_u,'-or');
legend('x(t)'); grid on;
subplot(224);plot(pi_v,'-or');
legend('pi(t)'); grid on;









